<?php

namespace AsyncAws\Sns\Exception;

use AsyncAws\Core\Exception\Http\ClientException;

/**
 * Exception error indicating platform application disabled.
 */
final class PlatformApplicationDisabledException extends ClientException
{
}
